#ifdef HAVE_DREAMBOX_HARDWARE
#include <png.h>
#include "my_lcd.h"

#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>


#ifdef USEFREETYPELCD
FT_Error LCD_FaceRequester(FTC_FaceID face_id, FT_Library library, FT_Pointer request_data, FT_Face *aface)
{
	return ((CLCDDisplay*)request_data)->FTC_Face_Requester(face_id, aface);
}

FT_Error CLCDDisplay::FTC_Face_Requester(FTC_FaceID face_id, FT_Face* aface)
{
	int error;
	if ((error=FT_New_Face(library, FONT, 0, aface)))
	{
		printf(" failed: %s", strerror(error));
		return error;
	}
	FT_Select_Charmap(*aface, ft_encoding_unicode);
	return 0;
}
#endif

CLCDDisplay::CLCDDisplay()
{
	available = false;
	int maxbytes=4*1024*1024;
	//open device
	if((fd = open(LCD_DEV,O_RDWR)) < 0)
	{
		perror(LCD_DEV); return;
	}
	//clear the display geht nicht
	if ( ioctl(fd,LCD_IOCTL_CLEAR) < 0)
	{
		perror("clear failed");return;
	}
	//graphic (binary) mode
	int i=LCD_MODE_BIN;
	if ( ioctl(fd,LCD_IOCTL_ASC_MODE,&i) < 0 )
	{
		perror("graphic mode failed");
		return;
	}

#ifdef USEFREETYPELCD
	FT_Error error;

	if (error = FT_Init_FreeType(&library))
		printf("[FONT] <FT_Init_FreeType failed with Errorcode 0x%.2X>", error);

	if((error = FTC_Manager_New(library, 10, 20, maxbytes, LCD_FaceRequester, this, &manager)))
	{
		printf("[FONT] <FTC_Manager_New failed with Errorcode 0x%.2X>\n", error);
		FT_Done_FreeType(library);
	}
	if((error = FTC_SBitCache_New(manager, &cache)))
	{
		printf("[FONT] <FTC_SBitCache_New failed with Errorcode 0x%.2X>\n", error);
		FTC_Manager_Done(manager);
		FT_Done_FreeType(library);
	}

	if((FTC_Manager_Lookup_Face(manager, (char*)FONT, &face)))
	{
		printf("[FONT] <FTC_Manager_Lookup_Face failed with Errorcode 0x%.2X>\n", error);
		FTC_Manager_Done(manager);
		FT_Done_FreeType(library);
	}

	use_kerning = FT_HAS_KERNING(face);

	desc.font.face_id = (char*)FONT;
#if FREETYPE_MAJOR  == 2 && FREETYPE_MINOR == 0
	desc.type = ftc_image_mono;
#else
	desc.flags = FT_LOAD_MONOCHROME;
#endif
#endif//USEFREETYPEFB

	printf("[LCD] available.\n");
	available = true;
}

CLCDDisplay::~CLCDDisplay()
{
	if(fd) close(fd);
#ifdef USEFREETYPEFB
	FTC_Manager_Done(manager);
	FT_Done_FreeType(library);
#endif
}

int CLCDDisplay::invalid_col (int x)
{
	if( x > LCD_COLS ) return -1;
	if( x < 0 ) return -1;
	return 0;
}

int CLCDDisplay::invalid_row (int y)
{
	if( y > LCD_ROWS * 8 ) return -1;
	if( y < 0) return -1;
	return 0;
}


void CLCDDisplay::convert_data ()
{
	unsigned int x, y, z;
	char tmp;

	for (x = 0; x < LCD_COLS; x++)
	{   
		for (y = 0; y < LCD_ROWS; y++)
		{
			tmp = 0;

			for (z = 0; z < 8; z++)
				if (raw[y * 8 + z][x] == 1)
					tmp |= (1 << z);

			lcd[y][x] = tmp;
		}
	}
}


void CLCDDisplay::clear()
{
	if(available)
	{
		draw_fill_rect (-1, -1, 120, 64, PIXEL_OFF);
		update();
	}
}

void CLCDDisplay::update()
{
	if(available)
	{
		convert_data();
		if ( write(fd, lcd, LCD_BUFFER_SIZE) < 0)
			perror("lcdd: CLCDDisplay::update(): write()");
	}
}

void CLCDDisplay::draw_point(const int x, const int y, const int state)
{
	if ((x < 0) || (x >= LCD_COLS) || (y < 0) || (y >= (LCD_ROWS * 8)))
		return;

	if (state == LCD_PIXEL_INV)
		raw[y][x] ^= 1;
	else
		raw[y][x] = state;
}


void CLCDDisplay::draw_line(const int x1, const int y1, const int x2, const int y2, const int state)
{
	if(!available)
		return;

	int dx = abs (x1 - x2);
	int dy = abs (y1 - y2);
	int x;
	int y;
	int End;
	int step;

	if ( dx > dy )
	{
		int	p = 2 * dy - dx;
		int	twoDy = 2 * dy;
		int	twoDyDx = 2 * (dy-dx);

		if ( x1 > x2 )
		{
			x = x2;
			y = y2;
			End = x1;
			step = y1 < y2 ? -1 : 1;
		}
		else
		{
			x = x1;
			y = y1;
			End = x2;
			step = y2 < y1 ? -1 : 1;
		}

		draw_point(x, y, state);

		while( x < End )
		{
			x++;
			if ( p < 0 )
				p += twoDy;
			else
			{
				y += step;
				p += twoDyDx;
			}
			draw_point(x, y, state);
		}
	}
	else
	{
		int	p = 2 * dx - dy;
		int	twoDx = 2 * dx;
		int	twoDxDy = 2 * (dx-dy);

		if ( y1 > y2 )
		{
			x = x2;
			y = y2;
			End = y1;
			step = x1 < x2 ? -1 : 1;
		}
		else
		{
			x = x1;
			y = y1;
			End = y2;
			step = x2 < x1 ? -1 : 1;
		}

		draw_point(x, y, state);

		while( y < End )
		{
			y++;
			if ( p < 0 )
				p += twoDx;
			else
			{
				x += step;
				p += twoDxDy;
			}
			draw_point(x, y, state);
		}
	}
}


void CLCDDisplay::draw_fill_rect (int left,int top,int right,int bottom,int state)
{
	if(!available)
		return;

	int x,y;
	for(x = left + 1;x < right;x++) {
		for(y = top + 1;y < bottom;y++) {
			draw_point(x,y,state);
		}
	}
}

bool CLCDDisplay::load_png(const char * const filename)
{
	if(!available)
		return -1;

	png_structp  png_ptr;
	png_infop    info_ptr;
	unsigned int i;
	unsigned int pass;
	unsigned int number_passes;
	int          bit_depth;
	int          color_type;
	int          interlace_type;
	png_uint_32  width;
	png_uint_32  height;
	png_byte *   fbptr;
	FILE *       fh;
	bool         ret_value = false;

	if ((fh = fopen(filename, "rb")))
	{
		if ((png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL)))
		{
			if (!(info_ptr = png_create_info_struct(png_ptr)))
				png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
			else
			{
				if (!(setjmp(png_ptr->jmpbuf)))
				{
					png_init_io(png_ptr,fh);

					png_read_info(png_ptr, info_ptr);
					png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, NULL, NULL);

					if (
						(color_type == PNG_COLOR_TYPE_PALETTE) &&
						(bit_depth  == 1                     ) &&
						(width      == LCD_COLS              ) &&
						(height     == (LCD_ROWS * 8))
						)
					{
						png_set_packing(png_ptr); /* expand to 1 byte blocks */
						
						number_passes = png_set_interlace_handling(png_ptr);
						png_read_update_info(png_ptr,info_ptr);

						if (width == png_get_rowbytes(png_ptr, info_ptr))
						{
							ret_value = true;

							for (pass = 0; pass < number_passes; pass++)
							{
								fbptr = (png_byte *)raw;
								for (i = 0; i < height; i++, fbptr += width)
									png_read_row(png_ptr, fbptr, NULL);
							}
							png_read_end(png_ptr, info_ptr);
						}
					}
				}
				png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
			}
		}
		fclose(fh);
	}
	return ret_value;
}

#ifdef USEFREETYPELCD
void CLCDDisplay::RenderString(std::string word, int sx, int sy, int maxwidth, int layout, int size, int state)
{
	int stringlen, ex, charwidth;
	unsigned char *string = ((unsigned char*)word.c_str());

	desc.font.pix_width = desc.font.pix_height = size;

	if(layout != LEFT)
	{
		stringlen = GetStringLen(string);
		switch(layout)
		{
			case CENTER:	if(stringlen < maxwidth) sx += (maxwidth - stringlen)/2;
					break;

			case RIGHT:	if(stringlen < maxwidth) sx += maxwidth - stringlen;
					break;
		}
	}
	//reset kerning
	prev_glyphindex = 0;
	//render string
	ex = sx + maxwidth;
	while(*string != '\0')
	{
		if((charwidth = RenderChar(*string, sx, sy, ex, state)) == -1) return; /* string > maxwidth */
		sx += charwidth;
		string++;
	}
}

int CLCDDisplay::GetStringLen(unsigned char *string)
{
	int stringlen = 0;
	//reset kerning
	prev_glyphindex = 0;
	//calc len
	while(*string != '\0')
	{
		stringlen += RenderChar(*string, -1, -1, -1, -1);
		string++;
	}
	return stringlen;
}

int CLCDDisplay::RenderChar(FT_ULong currentchar, int sx, int sy, int ex, int state)
{
	int x = 0, y = 0;
	FT_Error error;
	FT_UInt glyphindex;
	FT_Vector kerning;
	FTC_Node anode;

	//load char
	if(!(glyphindex = FT_Get_Char_Index(face, currentchar)))
	{
		printf("TuxMail <FT_Get_Char_Index for Char \"%c\" failed: \"undefined character code\">\n", (int)currentchar);return 0;
	}

	if((error = FTC_SBitCache_Lookup(cache, &desc, glyphindex, &sbit, &anode)))
	{
		printf("TuxMail <FTC_SBitCache_Lookup for Char \"%c\" failed with Errorcode 0x%.2X>\n", (int)currentchar, error);return 0;
	}

	if(use_kerning)
	{
		FT_Get_Kerning(face, prev_glyphindex, glyphindex, ft_kerning_default, &kerning);
		prev_glyphindex = glyphindex;
		kerning.x >>= 6;
	}
	else kerning.x = 0;

	//render char
	if(state != -1)
	{
		if(sx + sbit->xadvance >= ex) return -1;

		for(int row = 0; row < sbit->height; row++)
		{
			for(int pitch = 0; pitch < sbit->pitch; pitch++)
			{
				for(int bit = 7; bit >= 0; bit--)
				{
					if(pitch*8 + 7-bit >= sbit->width) break;

					if((sbit->buffer[row * sbit->pitch + pitch]) & 1<<bit)
					{
						draw_point((sx + sbit->left + kerning.x + x),(sy - sbit->top + y)+10,state);
					}
					x++;
				}
			}
			x = 0;
			y++;
		}
	}

	return sbit->xadvance + kerning.x;
}

#else

#define FONTDATAMAX 2048

static unsigned char font[FONTDATAMAX] = {

   /* 0 0x00 '^@' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 1 0x01 '^A' */
   0x7e, /* 01111110 */
   0x81, /* 10000001 */
   0xa5, /* 10100101 */
   0x81, /* 10000001 */
   0xbd, /* 10111101 */
   0x99, /* 10011001 */
   0x81, /* 10000001 */
   0x7e, /* 01111110 */

   /* 2 0x02 '^B' */
   0x7e, /* 01111110 */
   0xff, /* 11111111 */
   0xdb, /* 11011011 */
   0xff, /* 11111111 */
   0xc3, /* 11000011 */
   0xe7, /* 11100111 */
   0xff, /* 11111111 */
   0x7e, /* 01111110 */

   /* 3 0x03 '^C' */
   0x6c, /* 01101100 */
   0xfe, /* 11111110 */
   0xfe, /* 11111110 */
   0xfe, /* 11111110 */
   0x7c, /* 01111100 */
   0x38, /* 00111000 */
   0x10, /* 00010000 */
   0x00, /* 00000000 */

   /* 4 0x04 '^D' */
   0x10, /* 00010000 */
   0x38, /* 00111000 */
   0x7c, /* 01111100 */
   0xfe, /* 11111110 */
   0x7c, /* 01111100 */
   0x38, /* 00111000 */
   0x10, /* 00010000 */
   0x00, /* 00000000 */

   /* 5 0x05 '^E' */
   0x38, /* 00111000 */
   0x7c, /* 01111100 */
   0x38, /* 00111000 */
   0xfe, /* 11111110 */
   0xfe, /* 11111110 */
   0xd6, /* 11010110 */
   0x10, /* 00010000 */
   0x38, /* 00111000 */

   /* 6 0x06 '^F' */
   0x10, /* 00010000 */
   0x38, /* 00111000 */
   0x7c, /* 01111100 */
   0xfe, /* 11111110 */
   0xfe, /* 11111110 */
   0x7c, /* 01111100 */
   0x10, /* 00010000 */
   0x38, /* 00111000 */

   /* 7 0x07 '^G' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 8 0x08 '^H' */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xe7, /* 11100111 */
   0xc3, /* 11000011 */
   0xc3, /* 11000011 */
   0xe7, /* 11100111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */

   /* 9 0x09 '^I' */
   0x00, /* 00000000 */
   0x3c, /* 00111100 */
   0x66, /* 01100110 */
   0x42, /* 01000010 */
   0x42, /* 01000010 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x00, /* 00000000 */

   /* 10 0x0a '^J' */
   0xff, /* 11111111 */
   0xc3, /* 11000011 */
   0x99, /* 10011001 */
   0xbd, /* 10111101 */
   0xbd, /* 10111101 */
   0x99, /* 10011001 */
   0xc3, /* 11000011 */
   0xff, /* 11111111 */

   /* 11 0x0b '^K' */
   0x0f, /* 00001111 */
   0x07, /* 00000111 */
   0x0f, /* 00001111 */
   0x7d, /* 01111101 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0x78, /* 01111000 */

   /* 12 0x0c '^L' */
   0x3c, /* 00111100 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */

   /* 13 0x0d '^M' */
   0x3f, /* 00111111 */
   0x33, /* 00110011 */
   0x3f, /* 00111111 */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x70, /* 01110000 */
   0xf0, /* 11110000 */
   0xe0, /* 11100000 */

   /* 14 0x0e '^N' */
   0x7f, /* 01111111 */
   0x63, /* 01100011 */
   0x7f, /* 01111111 */
   0x63, /* 01100011 */
   0x63, /* 01100011 */
   0x67, /* 01100111 */
   0xe6, /* 11100110 */
   0xc0, /* 11000000 */

   /* 15 0x0f '^O' */
   0x18, /* 00011000 */
   0xdb, /* 11011011 */
   0x3c, /* 00111100 */
   0xe7, /* 11100111 */
   0xe7, /* 11100111 */
   0x3c, /* 00111100 */
   0xdb, /* 11011011 */
   0x18, /* 00011000 */

   /* 16 0x10 '^P' */
   0x80, /* 10000000 */
   0xe0, /* 11100000 */
   0xf8, /* 11111000 */
   0xfe, /* 11111110 */
   0xf8, /* 11111000 */
   0xe0, /* 11100000 */
   0x80, /* 10000000 */
   0x00, /* 00000000 */

   /* 17 0x11 '^Q' */
   0x02, /* 00000010 */
   0x0e, /* 00001110 */
   0x3e, /* 00111110 */
   0xfe, /* 11111110 */
   0x3e, /* 00111110 */
   0x0e, /* 00001110 */
   0x02, /* 00000010 */
   0x00, /* 00000000 */

   /* 18 0x12 '^R' */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */

   /* 19 0x13 '^S' */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x00, /* 00000000 */
   0x66, /* 01100110 */
   0x00, /* 00000000 */

   /* 20 0x14 '^T' */
   0x7f, /* 01111111 */
   0xdb, /* 11011011 */
   0xdb, /* 11011011 */
   0x7b, /* 01111011 */
   0x1b, /* 00011011 */
   0x1b, /* 00011011 */
   0x1b, /* 00011011 */
   0x00, /* 00000000 */

   /* 21 0x15 '^U' */
   0x3e, /* 00111110 */
   0x61, /* 01100001 */
   0x3c, /* 00111100 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x86, /* 10000110 */
   0x7c, /* 01111100 */

   /* 22 0x16 '^V' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x7e, /* 01111110 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */

   /* 23 0x17 '^W' */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0xff, /* 11111111 */

   /* 24 0x18 '^X' */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 25 0x19 '^Y' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 26 0x1a '^Z' */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0xfe, /* 11111110 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 27 0x1b '^[' */
   0x00, /* 00000000 */
   0x30, /* 00110000 */
   0x60, /* 01100000 */
   0xfe, /* 11111110 */
   0x60, /* 01100000 */
   0x30, /* 00110000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 28 0x1c '^\' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 29 0x1d '^]' */
   0x00, /* 00000000 */
   0x24, /* 00100100 */
   0x66, /* 01100110 */
   0xff, /* 11111111 */
   0x66, /* 01100110 */
   0x24, /* 00100100 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 30 0x1e '^^' */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x7e, /* 01111110 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 31 0x1f '^_' */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0x7e, /* 01111110 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 32 0x20 ' ' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 33 0x21 '!' */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x3c, /* 00111100 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 34 0x22 '"' */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 35 0x23 '#' */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0xfe, /* 11111110 */
   0x6c, /* 01101100 */
   0xfe, /* 11111110 */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0x00, /* 00000000 */

   /* 36 0x24 '$' */
   0x18, /* 00011000 */
   0x3e, /* 00111110 */
   0x60, /* 01100000 */
   0x3c, /* 00111100 */
   0x06, /* 00000110 */
   0x7c, /* 01111100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 37 0x25 '%' */
   0x00, /* 00000000 */
   0xc6, /* 11000110 */
   0xcc, /* 11001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x66, /* 01100110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 38 0x26 '&' */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0x68, /* 01101000 */
   0x76, /* 01110110 */
   0xdc, /* 11011100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 39 0x27 ''' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 40 0x28 '(' */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0x00, /* 00000000 */

   /* 41 0x29 ')' */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0x0c, /* 00001100 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x00, /* 00000000 */

   /* 42 0x2a '*' */
   0x00, /* 00000000 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0xff, /* 11111111 */
   0x3c, /* 00111100 */
   0x66, /* 01100110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 43 0x2b '+' */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 44 0x2c ',' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */

   /* 45 0x2d '-' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 46 0x2e '.' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 47 0x2f '/' */
   0x03, /* 00000011 */
   0x06, /* 00000110 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x60, /* 01100000 */
   0xc0, /* 11000000 */
   0x00, /* 00000000 */

   /* 48 0x30 '0' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xde, /* 11011110 */
   0xfe, /* 11111110 */
   0xf6, /* 11110110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 49 0x31 '1' */
   0x18, /* 00011000 */
   0x78, /* 01111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 50 0x32 '2' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x60, /* 01100000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */

   /* 51 0x33 '3' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0x06, /* 00000110 */
   0x1c, /* 00011100 */
   0x06, /* 00000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 52 0x34 '4' */
   0x1c, /* 00011100 */
   0x3c, /* 00111100 */
   0x6c, /* 01101100 */
   0xcc, /* 11001100 */
   0xfe, /* 11111110 */
   0x0c, /* 00001100 */
   0x0c, /* 00001100 */
   0x00, /* 00000000 */

   /* 53 0x35 '5' */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0xfc, /* 11111100 */
   0x06, /* 00000110 */
   0x06, /* 00000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 54 0x36 '6' */
   0x38, /* 00111000 */
   0x60, /* 01100000 */
   0xc0, /* 11000000 */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 55 0x37 '7' */
   0xfe, /* 11111110 */
   0x06, /* 00000110 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x60, /* 01100000 */
   0x60, /* 01100000 */
   0x00, /* 00000000 */

   /* 56 0x38 '8' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 57 0x39 '9' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7e, /* 01111110 */
   0x06, /* 00000110 */
   0x0c, /* 00001100 */
   0x38, /* 00111000 */
   0x00, /* 00000000 */

   /* 58 0x3a ':' */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 59 0x3b ';' */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */

   /* 60 0x3c '<' */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x60, /* 01100000 */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0x00, /* 00000000 */

   /* 61 0x3d '=' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 62 0x3e '>' */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0x06, /* 00000110 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x00, /* 00000000 */

   /* 63 0x3f '?' */
   0x3c, /* 00111100 */
   0x66, /* 01100110 */
   0x06, /* 00000110 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 64 0x40 '@' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xde, /* 11011110 */
   0xde, /* 11011110 */
   0xde, /* 11011110 */
   0xc0, /* 11000000 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 65 0x41 'A' */
   0x10, /* 00010000 */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 66 0x42 'B' */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xfc, /* 11111100 */
   0x00, /* 00000000 */

   /* 67 0x43 'C' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 68 0x44 'D' */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xfc, /* 11111100 */
   0x00, /* 00000000 */

   /* 69 0x45 'E' */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xf8, /* 11111000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */

   /* 70 0x46 'F' */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xf8, /* 11111000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0x00, /* 00000000 */

   /* 71 0x47 'G' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc0, /* 11000000 */
   0xce, /* 11001110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 72 0x48 'H' */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 73 0x49 'I' */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */

   /* 74 0x4a 'J' */
   0x06, /* 00000110 */
   0x06, /* 00000110 */
   0x06, /* 00000110 */
   0x06, /* 00000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 75 0x4b 'K' */
   0xc6, /* 11000110 */
   0xcc, /* 11001100 */
   0xd8, /* 11011000 */
   0xf0, /* 11110000 */
   0xd8, /* 11011000 */
   0xcc, /* 11001100 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 76 0x4c 'L' */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */

   /* 77 0x4d 'M' */
   0x82, /* 10000010 */
   0xc6, /* 11000110 */
   0xee, /* 11101110 */
   0xfe, /* 11111110 */
   0xd6, /* 11010110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 78 0x4e 'N' */
   0xc6, /* 11000110 */
   0xe6, /* 11100110 */
   0xf6, /* 11110110 */
   0xde, /* 11011110 */
   0xce, /* 11001110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 79 0x4f 'O' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 80 0x50 'P' */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xfc, /* 11111100 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0x00, /* 00000000 */

   /* 81 0x51 'Q' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xf6, /* 11110110 */
   0xde, /* 11011110 */
   0x7c, /* 01111100 */
   0x06, /* 00000110 */

   /* 82 0x52 'R' */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xfc, /* 11111100 */
   0xd8, /* 11011000 */
   0xcc, /* 11001100 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 83 0x53 'S' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0x60, /* 01100000 */
   0x38, /* 00111000 */
   0x0c, /* 00001100 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 84 0x54 'T' */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 85 0x55 'U' */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 86 0x56 'V' */
   0xc3, /* 11000011 */
   0xc3, /* 11000011 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 87 0x57 'W' */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xd6, /* 11010110 */
   0xfe, /* 11111110 */
   0xee, /* 11101110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 88 0x58 'X' */
   0xc3, /* 11000011 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x66, /* 01100110 */
   0xc3, /* 11000011 */
   0x00, /* 00000000 */

   /* 89 0x59 'Y' */
   0xc3, /* 11000011 */
   0xc3, /* 11000011 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 90 0x5a 'Z' */
   0xfe, /* 11111110 */
   0x06, /* 00000110 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x60, /* 01100000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */

   /* 91 0x5b '[' */
   0x3c, /* 00111100 */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x3c, /* 00111100 */
   0x00, /* 00000000 */

   /* 92 0x5c '\' */
   0xc0, /* 11000000 */
   0x60, /* 01100000 */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0x06, /* 00000110 */
   0x03, /* 00000011 */
   0x00, /* 00000000 */

   /* 93 0x5d ']' */
   0x3c, /* 00111100 */
   0x0c, /* 00001100 */
   0x0c, /* 00001100 */
   0x0c, /* 00001100 */
   0x0c, /* 00001100 */
   0x0c, /* 00001100 */
   0x3c, /* 00111100 */
   0x00, /* 00000000 */

   /* 94 0x5e '^' */
   0x10, /* 00010000 */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 95 0x5f '_' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */

   /* 96 0x60 '`' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 97 0x61 'a' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7c, /* 01111100 */
   0x06, /* 00000110 */
   0x7e, /* 01111110 */
   0xc6, /* 11000110 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */

   /* 98 0x62 'b' */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xfc, /* 11111100 */
   0x00, /* 00000000 */

   /* 99 0x63 'c' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc0, /* 11000000 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 100 0x64 'd' */
   0x06, /* 00000110 */
   0x06, /* 00000110 */
   0x7e, /* 01111110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */

   /* 101 0x65 'e' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 102 0x66 'f' */
   0x3c, /* 00111100 */
   0x66, /* 01100110 */
   0x60, /* 01100000 */
   0xf0, /* 11110000 */
   0x60, /* 01100000 */
   0x60, /* 01100000 */
   0x60, /* 01100000 */
   0x00, /* 00000000 */

   /* 103 0x67 'g' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7e, /* 01111110 */
   0x06, /* 00000110 */
   0x7c, /* 01111100 */

   /* 104 0x68 'h' */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 105 0x69 'i' */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x38, /* 00111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 106 0x6a 'j' */
   0x06, /* 00000110 */
   0x00, /* 00000000 */
   0x06, /* 00000110 */
   0x06, /* 00000110 */
   0x06, /* 00000110 */
   0x06, /* 00000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */

   /* 107 0x6b 'k' */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xcc, /* 11001100 */
   0xd8, /* 11011000 */
   0xf0, /* 11110000 */
   0xd8, /* 11011000 */
   0xcc, /* 11001100 */
   0x00, /* 00000000 */

   /* 108 0x6c 'l' */
   0x38, /* 00111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 109 0x6d 'm' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xec, /* 11101100 */
   0xfe, /* 11111110 */
   0xd6, /* 11010110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 110 0x6e 'n' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 111 0x6f 'o' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 112 0x70 'p' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xfc, /* 11111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xfc, /* 11111100 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */

   /* 113 0x71 'q' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7e, /* 01111110 */
   0x06, /* 00000110 */
   0x06, /* 00000110 */

   /* 114 0x72 'r' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xdc, /* 11011100 */
   0xe6, /* 11100110 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0x00, /* 00000000 */

   /* 115 0x73 's' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0xc0, /* 11000000 */
   0x7c, /* 01111100 */
   0x06, /* 00000110 */
   0xfc, /* 11111100 */
   0x00, /* 00000000 */

   /* 116 0x74 't' */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x7c, /* 01111100 */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x36, /* 00110110 */
   0x1c, /* 00011100 */
   0x00, /* 00000000 */

   /* 117 0x75 'u' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 118 0x76 'v' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x6c, /* 01101100 */
   0x38, /* 00111000 */
   0x00, /* 00000000 */

   /* 119 0x77 'w' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xd6, /* 11010110 */
   0xfe, /* 11111110 */
   0x6c, /* 01101100 */
   0x00, /* 00000000 */

   /* 120 0x78 'x' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xc6, /* 11000110 */
   0x6c, /* 01101100 */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 121 0x79 'y' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xc3, /* 11000011 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x60, /* 01100000 */

   /* 122 0x7a 'z' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */
   0x0c, /* 00001100 */
   0x38, /* 00111000 */
   0x60, /* 01100000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */

   /* 123 0x7b '{' */
   0x0e, /* 00001110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x70, /* 01110000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x0e, /* 00001110 */
   0x00, /* 00000000 */

   /* 124 0x7c '|' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 125 0x7d '}' */
   0x70, /* 01110000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x0e, /* 00001110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x70, /* 01110000 */
   0x00, /* 00000000 */

   /* 126 0x7e '~' */
   0x72, /* 01110010 */
   0x9c, /* 10011100 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 127 0x7f '' */
   0x00, /* 00000000 */
   0x10, /* 00010000 */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */

   /* 128 0x80 '' */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x0c, /* 00001100 */
   0x78, /* 01111000 */

   /* 129 0x81 '' */
   0xcc, /* 11001100 */
   0x00, /* 00000000 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 130 0x82 '' */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 131 0x83 '' */
   0x7c, /* 01111100 */
   0x82, /* 10000010 */
   0x78, /* 01111000 */
   0x0c, /* 00001100 */
   0x7c, /* 01111100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 132 0x84 '' */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */
   0x78, /* 01111000 */
   0x0c, /* 00001100 */
   0x7c, /* 01111100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 133 0x85 '' */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x78, /* 01111000 */
   0x0c, /* 00001100 */
   0x7c, /* 01111100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 134 0x86 '' */
   0x30, /* 00110000 */
   0x30, /* 00110000 */
   0x78, /* 01111000 */
   0x0c, /* 00001100 */
   0x7c, /* 01111100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 135 0x87 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0x7e, /* 01111110 */
   0x0c, /* 00001100 */
   0x38, /* 00111000 */

   /* 136 0x88 '' */
   0x7c, /* 01111100 */
   0x82, /* 10000010 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 137 0x89 '' */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 138 0x8a '' */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 139 0x8b '' */
   0x66, /* 01100110 */
   0x00, /* 00000000 */
   0x38, /* 00111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x00, /* 00000000 */

   /* 140 0x8c '' */
   0x7c, /* 01111100 */
   0x82, /* 10000010 */
   0x38, /* 00111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x00, /* 00000000 */

   /* 141 0x8d '' */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x38, /* 00111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x00, /* 00000000 */

   /* 142 0x8e '' */
   0xc6, /* 11000110 */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 143 0x8f '' */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 144 0x90 '' */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0xf8, /* 11111000 */
   0xc0, /* 11000000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */

   /* 145 0x91 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0xd8, /* 11011000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */

   /* 146 0x92 '' */
   0x3e, /* 00111110 */
   0x6c, /* 01101100 */
   0xcc, /* 11001100 */
   0xfe, /* 11111110 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xce, /* 11001110 */
   0x00, /* 00000000 */

   /* 147 0x93 '' */
   0x7c, /* 01111100 */
   0x82, /* 10000010 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 148 0x94 '' */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 149 0x95 '' */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 150 0x96 '' */
   0x78, /* 01111000 */
   0x84, /* 10000100 */
   0x00, /* 00000000 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 151 0x97 '' */
   0x60, /* 01100000 */
   0x30, /* 00110000 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 152 0x98 '' */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7e, /* 01111110 */
   0x06, /* 00000110 */
   0xfc, /* 11111100 */

   /* 153 0x99 '' */
   0xc6, /* 11000110 */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x6c, /* 01101100 */
   0x38, /* 00111000 */
   0x00, /* 00000000 */

   /* 154 0x9a '' */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 155 0x9b '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 156 0x9c '' */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0x64, /* 01100100 */
   0xf0, /* 11110000 */
   0x60, /* 01100000 */
   0x66, /* 01100110 */
   0xfc, /* 11111100 */
   0x00, /* 00000000 */

   /* 157 0x9d '' */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 158 0x9e '' */
   0xf8, /* 11111000 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xfa, /* 11111010 */
   0xc6, /* 11000110 */
   0xcf, /* 11001111 */
   0xc6, /* 11000110 */
   0xc7, /* 11000111 */

   /* 159 0x9f '' */
   0x0e, /* 00001110 */
   0x1b, /* 00011011 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0xd8, /* 11011000 */
   0x70, /* 01110000 */
   0x00, /* 00000000 */

   /* 160 0xa0 '' */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x78, /* 01111000 */
   0x0c, /* 00001100 */
   0x7c, /* 01111100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 161 0xa1 '' */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x38, /* 00111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x00, /* 00000000 */

   /* 162 0xa2 '' */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */

   /* 163 0xa3 '' */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 164 0xa4 '' */
   0x76, /* 01110110 */
   0xdc, /* 11011100 */
   0x00, /* 00000000 */
   0xdc, /* 11011100 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x00, /* 00000000 */

   /* 165 0xa5 '' */
   0x76, /* 01110110 */
   0xdc, /* 11011100 */
   0x00, /* 00000000 */
   0xe6, /* 11100110 */
   0xf6, /* 11110110 */
   0xde, /* 11011110 */
   0xce, /* 11001110 */
   0x00, /* 00000000 */

   /* 166 0xa6 '' */
   0x3c, /* 00111100 */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0x3e, /* 00111110 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 167 0xa7 '' */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0x38, /* 00111000 */
   0x00, /* 00000000 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 168 0xa8 '' */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x63, /* 01100011 */
   0x3e, /* 00111110 */
   0x00, /* 00000000 */

   /* 169 0xa9 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 170 0xaa '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */
   0x06, /* 00000110 */
   0x06, /* 00000110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 171 0xab '' */
   0x63, /* 01100011 */
   0xe6, /* 11100110 */
   0x6c, /* 01101100 */
   0x7e, /* 01111110 */
   0x33, /* 00110011 */
   0x66, /* 01100110 */
   0xcc, /* 11001100 */
   0x0f, /* 00001111 */

   /* 172 0xac '' */
   0x63, /* 01100011 */
   0xe6, /* 11100110 */
   0x6c, /* 01101100 */
   0x7a, /* 01111010 */
   0x36, /* 00110110 */
   0x6a, /* 01101010 */
   0xdf, /* 11011111 */
   0x06, /* 00000110 */

   /* 173 0xad '' */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 174 0xae '' */
   0x00, /* 00000000 */
   0x33, /* 00110011 */
   0x66, /* 01100110 */
   0xcc, /* 11001100 */
   0x66, /* 01100110 */
   0x33, /* 00110011 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 175 0xaf '' */
   0x00, /* 00000000 */
   0xcc, /* 11001100 */
   0x66, /* 01100110 */
   0x33, /* 00110011 */
   0x66, /* 01100110 */
   0xcc, /* 11001100 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 176 0xb0 '' */
   0x22, /* 00100010 */
   0x88, /* 10001000 */
   0x22, /* 00100010 */
   0x88, /* 10001000 */
   0x22, /* 00100010 */
   0x88, /* 10001000 */
   0x22, /* 00100010 */
   0x88, /* 10001000 */

   /* 177 0xb1 '' */
   0x55, /* 01010101 */
   0xaa, /* 10101010 */
   0x55, /* 01010101 */
   0xaa, /* 10101010 */
   0x55, /* 01010101 */
   0xaa, /* 10101010 */
   0x55, /* 01010101 */
   0xaa, /* 10101010 */

   /* 178 0xb2 '' */
   0x77, /* 01110111 */
   0xdd, /* 11011101 */
   0x77, /* 01110111 */
   0xdd, /* 11011101 */
   0x77, /* 01110111 */
   0xdd, /* 11011101 */
   0x77, /* 01110111 */
   0xdd, /* 11011101 */

   /* 179 0xb3 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 180 0xb4 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0xf8, /* 11111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 181 0xb5 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0xf8, /* 11111000 */
   0x18, /* 00011000 */
   0xf8, /* 11111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 182 0xb6 '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0xf6, /* 11110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 183 0xb7 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 184 0xb8 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xf8, /* 11111000 */
   0x18, /* 00011000 */
   0xf8, /* 11111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 185 0xb9 '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0xf6, /* 11110110 */
   0x06, /* 00000110 */
   0xf6, /* 11110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 186 0xba '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 187 0xbb '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */
   0x06, /* 00000110 */
   0xf6, /* 11110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 188 0xbc '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0xf6, /* 11110110 */
   0x06, /* 00000110 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 189 0xbd '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 190 0xbe '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0xf8, /* 11111000 */
   0x18, /* 00011000 */
   0xf8, /* 11111000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 191 0xbf '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xf8, /* 11111000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 192 0xc0 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x1f, /* 00011111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 193 0xc1 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 194 0xc2 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 195 0xc3 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x1f, /* 00011111 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 196 0xc4 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 197 0xc5 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0xff, /* 11111111 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 198 0xc6 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x1f, /* 00011111 */
   0x18, /* 00011000 */
   0x1f, /* 00011111 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 199 0xc7 '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x37, /* 00110111 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 200 0xc8 '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x37, /* 00110111 */
   0x30, /* 00110000 */
   0x3f, /* 00111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 201 0xc9 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x3f, /* 00111111 */
   0x30, /* 00110000 */
   0x37, /* 00110111 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 202 0xca '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0xf7, /* 11110111 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 203 0xcb '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0xf7, /* 11110111 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 204 0xcc '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x37, /* 00110111 */
   0x30, /* 00110000 */
   0x37, /* 00110111 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 205 0xcd '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 206 0xce '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0xf7, /* 11110111 */
   0x00, /* 00000000 */
   0xf7, /* 11110111 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 207 0xcf '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 208 0xd0 '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 209 0xd1 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 210 0xd2 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 211 0xd3 '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x3f, /* 00111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 212 0xd4 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x1f, /* 00011111 */
   0x18, /* 00011000 */
   0x1f, /* 00011111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 213 0xd5 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x1f, /* 00011111 */
   0x18, /* 00011000 */
   0x1f, /* 00011111 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 214 0xd6 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x3f, /* 00111111 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 215 0xd7 '' */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0xff, /* 11111111 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */

   /* 216 0xd8 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0xff, /* 11111111 */
   0x18, /* 00011000 */
   0xff, /* 11111111 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 217 0xd9 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0xf8, /* 11111000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 218 0xda '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x1f, /* 00011111 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 219 0xdb '' */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */

   /* 220 0xdc '' */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */
#if 0
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
#endif

   /* 221 0xdd '' */
   0xf0, /* 11110000 */
   0xf0, /* 11110000 */
   0xf0, /* 11110000 */
   0xf0, /* 11110000 */
   0xf0, /* 11110000 */
   0xf0, /* 11110000 */
   0xf0, /* 11110000 */
   0xf0, /* 11110000 */

   /* 222 0xde '' */
   0x0f, /* 00001111 */
   0x0f, /* 00001111 */
   0x0f, /* 00001111 */
   0x0f, /* 00001111 */
   0x0f, /* 00001111 */
   0x0f, /* 00001111 */
   0x0f, /* 00001111 */
   0x0f, /* 00001111 */

   /* 223 0xdf '' */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0xff, /* 11111111 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 224 0xe0 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x76, /* 01110110 */
   0xdc, /* 11011100 */
   0xc8, /* 11001000 */
   0xdc, /* 11011100 */
   0x76, /* 01110110 */
   0x00, /* 00000000 */

   /* 225 0xe1 '' */
   0x78, /* 01111000 */
   0xcc, /* 11001100 */
   0xcc, /* 11001100 */
   0xd8, /* 11011000 */
   0xcc, /* 11001100 */
   0xc6, /* 11000110 */
   0xcc, /* 11001100 */
   0x00, /* 00000000 */

   /* 226 0xe2 '' */
   0xfe, /* 11111110 */
   0xc6, /* 11000110 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0xc0, /* 11000000 */
   0x00, /* 00000000 */

   /* 227 0xe3 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0x00, /* 00000000 */

   /* 228 0xe4 '' */
   0xfe, /* 11111110 */
   0xc6, /* 11000110 */
   0x60, /* 01100000 */
   0x30, /* 00110000 */
   0x60, /* 01100000 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */

   /* 229 0xe5 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0xd8, /* 11011000 */
   0xd8, /* 11011000 */
   0xd8, /* 11011000 */
   0x70, /* 01110000 */
   0x00, /* 00000000 */

   /* 230 0xe6 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x7c, /* 01111100 */
   0xc0, /* 11000000 */

   /* 231 0xe7 '' */
   0x00, /* 00000000 */
   0x76, /* 01110110 */
   0xdc, /* 11011100 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */

   /* 232 0xe8 '' */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x3c, /* 00111100 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */

   /* 233 0xe9 '' */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0xc6, /* 11000110 */
   0xfe, /* 11111110 */
   0xc6, /* 11000110 */
   0x6c, /* 01101100 */
   0x38, /* 00111000 */
   0x00, /* 00000000 */

   /* 234 0xea '' */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0xee, /* 11101110 */
   0x00, /* 00000000 */

   /* 235 0xeb '' */
   0x0e, /* 00001110 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0x3e, /* 00111110 */
   0x66, /* 01100110 */
   0x66, /* 01100110 */
   0x3c, /* 00111100 */
   0x00, /* 00000000 */

   /* 236 0xec '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0xdb, /* 11011011 */
   0xdb, /* 11011011 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 237 0xed '' */
   0x06, /* 00000110 */
   0x0c, /* 00001100 */
   0x7e, /* 01111110 */
   0xdb, /* 11011011 */
   0xdb, /* 11011011 */
   0x7e, /* 01111110 */
   0x60, /* 01100000 */
   0xc0, /* 11000000 */

   /* 238 0xee '' */
   0x1e, /* 00011110 */
   0x30, /* 00110000 */
   0x60, /* 01100000 */
   0x7e, /* 01111110 */
   0x60, /* 01100000 */
   0x30, /* 00110000 */
   0x1e, /* 00011110 */
   0x00, /* 00000000 */

   /* 239 0xef '' */
   0x00, /* 00000000 */
   0x7c, /* 01111100 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0xc6, /* 11000110 */
   0x00, /* 00000000 */

   /* 240 0xf0 '' */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */
   0xfe, /* 11111110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 241 0xf1 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x7e, /* 01111110 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */

   /* 242 0xf2 '' */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */

   /* 243 0xf3 '' */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x18, /* 00011000 */
   0x0c, /* 00001100 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */

   /* 244 0xf4 '' */
   0x0e, /* 00001110 */
   0x1b, /* 00011011 */
   0x1b, /* 00011011 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */

   /* 245 0xf5 '' */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0xd8, /* 11011000 */
   0xd8, /* 11011000 */
   0x70, /* 01110000 */

   /* 246 0xf6 '' */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x7e, /* 01111110 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 247 0xf7 '' */
   0x00, /* 00000000 */
   0x76, /* 01110110 */
   0xdc, /* 11011100 */
   0x00, /* 00000000 */
   0x76, /* 01110110 */
   0xdc, /* 11011100 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 248 0xf8 '' */
   0x38, /* 00111000 */
   0x6c, /* 01101100 */
   0x6c, /* 01101100 */
   0x38, /* 00111000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 249 0xf9 '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 250 0xfa '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x18, /* 00011000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 251 0xfb '' */
   0x0f, /* 00001111 */
   0x0c, /* 00001100 */
   0x0c, /* 00001100 */
   0x0c, /* 00001100 */
   0xec, /* 11101100 */
   0x6c, /* 01101100 */
   0x3c, /* 00111100 */
   0x1c, /* 00011100 */

   /* 252 0xfc '' */
   0x6c, /* 01101100 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x36, /* 00110110 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 253 0xfd '' */
   0x78, /* 01111000 */
   0x0c, /* 00001100 */
   0x18, /* 00011000 */
   0x30, /* 00110000 */
   0x7c, /* 01111100 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 254 0xfe '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x3c, /* 00111100 */
   0x3c, /* 00111100 */
   0x3c, /* 00111100 */
   0x3c, /* 00111100 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

   /* 255 0xff '' */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */
   0x00, /* 00000000 */

};

void CLCDDisplay::draw_char(int x, int y, char c)
{
	unsigned char *data=&font[8*c];
	for (int ay=0; ay<8; ay++)
		for (int ax=0; ax<8; ax++)
			if ( (!invalid_col(x+ax)) && (!invalid_row(y+ay)))
				draw_point(x+ax, y+ay, data[ay]&(1<<(7-ax))?PIXEL_ON:PIXEL_OFF);
}

void CLCDDisplay::draw_string(int x, int y, const char *string)
{
	if(!available)
		return;

	while (*string)
	{
		draw_char(x, y, *string++);
		x+=8;
	}
}
#endif//USEFREETYPELCD
#endif
